@@ -112,4 +112,4 @@ def feature_control_rewards(
   elif cumulant_type == 'potential':
     return discount * features[1:] - features[:-1]
   else:
-    raise ValueError('Unknown cumulant_type {}'.format(cumulant_type))
+    raise ValueError(f'Unknown cumulant_type {cumulant_type}')
